<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Doctrine\ORM;

use DateTime;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Doctrine\ORM\QueryBuilder;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Component\Channel\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderItemUnitInterface;

class VoucherRepository extends EntityRepository implements VoucherRepositoryInterface
{
    public function createListQueryBuilder(): QueryBuilder
    {
        return $this->createQueryBuilder('o');
    }

    public function findOneEnabledNotExpiredByCodeAndChannel(string $code, ChannelInterface $channel): ?VoucherInterface
    {
        $qb = $this->createListQueryBuilder();

        $qb->andWhere('o.code = :code');
        $qb->andWhere('o.channel = :channel');
        $qb->andWhere('o.enabled = :enabled');
        $qb->andWhere('o.dateEnd <= :date_now');
        $qb->setParameter('channel', $channel);
        $qb->setParameter('code', $code);
        $qb->setParameter('enabled', true);
        $qb->setParameter('date_now', new DateTime());

        $query = $qb->getQuery();

        return $query->getOneOrNullResult();
    }

    public function findOneByCode(string $code): ?VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->findOneBy([
            'code' => $code,
        ]);

        return $voucher;
    }

    public function findOneByOrderItemUnit(OrderItemUnitInterface $orderItemUnit): ?VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->findOneBy([
            'orderItemUnit' => $orderItemUnit,
        ]);

        return $voucher;
    }

    public function findEnabledNotExpired(): array
    {
        $qb = $this->createListQueryBuilder();

        $this->enabledNotExpiredQueryBuilder($qb);

        $query = $qb->getQuery();

        return $query->getResult();
    }

    private function enabledNotExpiredQueryBuilder(QueryBuilder $queryBuilder, bool $enabled = true): QueryBuilder
    {
        return $queryBuilder
            ->andWhere('o.enabled = :enabled')
            ->andWhere('o.dateEnd >= :date_now')
            ->setParameter('date_now', new DateTime())
            ->setParameter(':enabled', $enabled)
        ;
    }

    public function createAccountListQueryBuilder(CustomerInterface $customer): QueryBuilder
    {
        $qb = $this->createListQueryBuilder();
        $qb->andWhere('o.customer = :customer');
        $qb->setParameter('customer', $customer);

        return $qb;
    }

    public function findEnabledByUserQueryBuilder(CustomerInterface $customer): QueryBuilder
    {
        $qb = $this->createListQueryBuilder();
        $qb->andWhere('o.customer = :customer');
        $qb->setParameter('customer', $customer);

        $this->enabledNotExpiredQueryBuilder($qb);

        return $qb;
    }

    public function findEnabledByUser(CustomerInterface $customer): array
    {
        return $this->findEnabledByUserQueryBuilder($customer)->getQuery()->getResult();
    }
}
