<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

trait OrderTrait
{
    /**
     * @var Collection|VoucherInterface[]
     *
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusVoucherPlugin\Model\VoucherInterface", inversedBy="appliedOrders")
     *
     * @ORM\JoinTable(name="dedi_sylius_voucher_order_vouchers",
     *     joinColumns={@ORM\JoinColumn(name="order_id", referencedColumnName="id", onDelete="CASCADE")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="voucher_id", referencedColumnName="id", onDelete="CASCADE")}
     * )
     */
    protected $vouchers;

    public function __construct()
    {
        $this->vouchers = new ArrayCollection();
    }

    public function getVouchers(): Collection
    {
        return $this->vouchers;
    }

    public function hasVouchers(): bool
    {
        return !$this->vouchers->isEmpty();
    }

    public function addVoucher(VoucherInterface $voucher): void
    {
        if (!$this->hasVoucher($voucher)) {
            $this->vouchers->add($voucher);
            $voucher->addAppliedOrder($this);
        }
    }

    public function removeVoucher(VoucherInterface $voucher): void
    {
        if ($this->hasVoucher($voucher)) {
            $this->vouchers->removeElement($voucher);
            $voucher->removeAppliedOrder($this);
        }
    }

    public function hasVoucher(VoucherInterface $voucher): bool
    {
        return $this->vouchers->contains($voucher);
    }
}
