<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Model;

use Countable;
use Iterator;
use OutOfBoundsException;
use Webmozart\Assert\Assert;

final class VoucherBalanceCollection implements Countable, Iterator
{
    /** @var VoucherBalance[] */
    private array $collection = [];

    public function addVoucher(VoucherInterface $voucher): void
    {
        $currencyCode = $voucher->getCurrencyCode();
        Assert::notNull($currencyCode);

        if (!isset($this->collection[$currencyCode])) {
            $this->collection[$currencyCode] = new VoucherBalance($currencyCode);
        }

        $this->collection[$currencyCode]->add($voucher->getAmount());
    }

    public static function createFromVouchers(iterable $vouchers): self
    {
        $collection = new self();
        foreach ($vouchers as $voucher) {
            $collection->addVoucher($voucher);
        }

        return $collection;
    }

    public function count(): int
    {
        return count($this->collection);
    }

    public function current(): VoucherBalance
    {
        $cur = current($this->collection);
        if (false === $cur) {
            throw new OutOfBoundsException('Do not call current on an uninitialized collection');
        }

        return $cur;
    }

    public function next(): void
    {
        next($this->collection);
    }

    public function key(): ?string
    {
        $k = (string) key($this->collection);

        return '' === $k ? null : $k;
    }

    public function valid(): bool
    {
        return key($this->collection) !== null;
    }

    public function rewind(): void
    {
        reset($this->collection);
    }
}
