<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Provider;

use Dedi\SyliusVoucherPlugin\Model\VoucherChannelConfigurationInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Component\Channel\Model\ChannelInterface;
use Sylius\Component\Locale\Context\LocaleContextInterface;
use Sylius\Component\Locale\Context\LocaleNotFoundException;
use Sylius\Component\Locale\Model\LocaleInterface;
use Sylius\Component\Order\Model\OrderInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Webmozart\Assert\Assert;

final class VoucherChannelConfigurationProvider implements VoucherChannelConfigurationProviderInterface
{
    private readonly RepositoryInterface $configurationRepository;

    private readonly RepositoryInterface $defaultConfigurationRepository;

    private readonly LocaleContextInterface $localeContext;

    private readonly RepositoryInterface $localeRepository;

    public function __construct(
        RepositoryInterface $configurationRepository,
        RepositoryInterface $defaultConfigurationRepository,
        LocaleContextInterface $localeContext,
        RepositoryInterface $localeRepository,
    ) {
        $this->configurationRepository = $configurationRepository;
        $this->defaultConfigurationRepository = $defaultConfigurationRepository;
        $this->localeContext = $localeContext;
        $this->localeRepository = $localeRepository;
    }

    public function getConfiguration(ChannelInterface $channel, LocaleInterface $locale): ?VoucherConfigurationInterface
    {
        $channelConfiguration = $this->configurationRepository->findOneBy(['channel' => $channel, 'locale' => $locale]);
        if ($channelConfiguration instanceof VoucherChannelConfigurationInterface) {
            /** @var VoucherConfigurationInterface $configuration */
            $configuration = $channelConfiguration->getConfiguration();
        } else {
            /** @var VoucherConfigurationInterface $configuration */
            $configuration = $this->defaultConfigurationRepository->findOneBy(['default' => true]);
        }

        return $configuration;
    }

    public function getConfigurationForVoucher(VoucherInterface $voucher): ?VoucherConfigurationInterface
    {
        $channel = $voucher->getChannel();
        Assert::isInstanceOf($channel, ChannelInterface::class);

        try {
            $order = $voucher->getOrder();
            if ($order instanceof OrderInterface) {
                $localeCode = $order->getLocaleCode();
            } else {
                $localeCode = $this->localeContext->getLocaleCode();
            }
            $locale = $this->localeRepository->findOneBy(['code' => $localeCode]);
            if (!$locale instanceof LocaleInterface) {
                throw new LocaleNotFoundException();
            }
        } catch (LocaleNotFoundException $exception) {
            $locale = $channel->getDefaultLocale();
        }

        return $this->getConfiguration($channel, $locale);
    }
}
